/*************************************************************
	ControlModule.c Ver.2.1

						2015.7.13 M.Ustumi@ARCDEVICE

**************************************************************/

#include"stdio.h"
#include"ControlModule.h"

volatile BYTE CM_bID = 0;
BYTE CM_bVersion = 0x21; // version 2.1

BYTE CM_bStatus = CM_NORMAL;
BYTE CM_bCtrlStatus = CM_NORMAL;

volatile WORD CM_wTimeCount=0;			//  JE^
volatile WORD CM_wCtrlTimeout = 1000;	//  ^CAEg
volatile WORD CM_wComTimeout = 1000;	//  ^CAEg

WORD _EEDATA(2) EEP_wID=1;	//vOݎ̏ ID
							//u[gvOꍇ͖
BYTE ReadID( void )
{
	WORD w;

	EEP_Read( 
		__builtin_tblpage(&EEP_wID),
		__builtin_tbloffset(&EEP_wID),
	 	&w, 1 );
	return( (BYTE)w );
}

void WriteID( BYTE bID )
{
	WORD w;

	w = bID;
	EEP_Write( 
		__builtin_tblpage(&EEP_wID),
		__builtin_tbloffset(&EEP_wID),
	 	&w, 1 );
}


void CM_Init( void )
{
	TRIS_LED = 0;

	CM_bStatus = CM_NORMAL;
	CM_bID = ReadID();
	
	USB_Open();
	USB_SetTimeout( CM_wComTimeout );

	ESPI1_Open();
	ESPI1_SetTimeout( CM_wComTimeout );

	UT_SetTimer( 1.0 ); // 1ms 
}


void CM_Reset( void )
{
	CM_Reply();
	asm( "RESET" );
}

void CM_Ping( void )
{
	BYTE b;

	b = CM_PING;
	if(!USB_WriteBuf( &b, 1 )){
		CM_bStatus = CM_COMMAND_TX_TIMEOUT;
		return;
	}
	CM_Reply();
}


void CM_SetID( void )
{
	BYTE b;
	if(!USB_Read( &b, 1 )){
		CM_bStatus = CM_COMMAND_RX_TIMEOUT;;
		return;
	}
	CM_Reply();

	WriteID( b );
	CM_bID = b;
}

void CM_GetID( void )
{
	if(!USB_WriteBuf( (BYTE*)&CM_bID, 1 )){
		CM_bStatus = CM_COMMAND_TX_TIMEOUT;
		return;
	}
	CM_Reply();
}

void CM_GetVer( void )
{
	if( !USB_WriteBuf( &CM_bVersion, 1 )){
		CM_bStatus = CM_REPLY_ERROR;
		return;
	}
	CM_Reply();
}

void CM_Blink( void )
{
	BYTE b;

	if(!USB_Read( &b, 1 )){
		CM_bStatus = CM_COMMAND_RX_TIMEOUT;
		return;
	}
	CM_Reply();

	UT_StatusBlink( 125, b, 4 );			
}

void SetTransferError( void )
{
	BYTE bDevStatus, bRxSize;

	bDevStatus = CM_TRANSFER_ERROR + ESPI1_bStatus;
	//CM_bStatus = CM_TRANSFER_ERROR;
	bRxSize = 2;
	if( !USB_WriteBuf( &bRxSize, 1 ) ){
		CM_bStatus = CM_COMMAND_TX_TIMEOUT;
		return ;
	}
	if( !USB_WriteBuf( &bDevStatus, 1 ) ){
		CM_bStatus = CM_COMMAND_TX_TIMEOUT;
		return ;
	}
	// dumy for checksum	
	if( !USB_WriteBuf( &bDevStatus, 1 ) ){
		CM_bStatus = CM_COMMAND_TX_TIMEOUT;
		return ;
	}
}

TR_HEADER Header;
WORD wRxBuffer[128];
WORD wTxBuffer[128];
WORD wRxSize;

BOOL SyncTransceive( void )
{
	BYTE *ptr, bCommand;
	WORD wWordSize, wRet;

// Start Hand shake
	bCommand = *(BYTE*)wTxBuffer;

	ESPI1_WriteEnable();
	if( !ESPI1_SetCommand( Header.bID, bCommand ) )	return( FALSE );

	ESPI1_WriteDisable();
	// ACK or NAK
	if( !ESPI1_ReadEnable() ) return( FALSE );
	
	if( !ESPI1_Read( &wRet, 1))	return( FALSE );
	
	ESPI1_ReadDisable();
// End Hand shake

	if( wRet == ACK ){
	// Transmit
		ESPI1_WriteEnable();
		
		if(!ESPI1_Write( &Header.wTxSize,  1 )) return( FALSE );
		UT_Wait_us(2);
		wWordSize = Header.wTxSize+1;
		wWordSize = wWordSize>>1;
		if( !ESPI1_Write( wTxBuffer, wWordSize ) )	return( FALSE );
		ESPI1_WriteDisable();

	// Receive

		if( !ESPI1_ReadEnable() ) return( FALSE );
		
		if( !ESPI1_Read( &wRxSize, 1) ) return( FALSE );
		wWordSize = wRxSize + 1;
		wWordSize = wWordSize>>1;
		if( !ESPI1_Read( wRxBuffer, wWordSize ) ) return( FALSE );
		ESPI1_ReadDisable();
	}else{ // NAK
		wRxSize = 2;
		ptr = (BYTE*)wRxBuffer;
		ptr[0] = CM_NAK;
		ptr[1] = CM_NAK;
		UT_Blink(2);

	}
	return( TRUE );
}

BOOL AsyncTransceive( void )
{
	BYTE bCommand;
	WORD wWordSize;

	bCommand = *(BYTE*)wTxBuffer;

	ESPI1_WriteEnable();
	if( !ESPI1_SetCommand( Header.bID, bCommand ) )	return( FALSE );
	ESPI1_WriteDisable();

// Receive
	if( !ESPI1_ReadEnable() ) return( FALSE );	
	if(!ESPI1_Read( &wRxSize, 1)) return( FALSE );
	
	wWordSize = wRxSize+1;
	wWordSize = wWordSize>>1;
	if(!ESPI1_Read( wRxBuffer, wWordSize)) return( FALSE );
	ESPI1_ReadDisable();

// Transmit
	ESPI1_WriteEnable();
		
	if(!ESPI1_Write( &Header.wTxSize,  1 )) return( FALSE );
	wWordSize = Header.wTxSize+1;
	wWordSize = wWordSize>>1;
	if( !ESPI1_Write( wTxBuffer, wWordSize ) )	return( FALSE );
	ESPI1_WriteDisable();

	return( TRUE );
}

void CM_Transfer( void )
{
	BYTE bComNum, bDevStatus, bCommand;
	WORD wWordSize, wSize;
	
	USB_InitTxBuffer();

	if(!USB_Read( &bComNum, 1 ) ){
		CM_bStatus = CM_COMMAND_RX_TIMEOUT;
	}else{
		while( bComNum-- ){
			if(!USB_Read( (BYTE*)&Header, sizeof( TR_HEADER ) )){
				CM_bStatus = CM_COMMAND_RX_TIMEOUT;
				return;
			}
			if( !USB_Read( (BYTE*)wTxBuffer, Header.wTxSize ) ){
				CM_bStatus = CM_COMMAND_RX_TIMEOUT;
				return;
			}
			if( !USB_WriteBuf( (BYTE*)&Header.bPort, 1 ) ){
				CM_bStatus = CM_COMMAND_TX_TIMEOUT;
				return;
			}
			if( !USB_WriteBuf( &Header.bID, 1 ) ){
				CM_bStatus = CM_COMMAND_TX_TIMEOUT;
				return;
			}

			bCommand = *(BYTE*)wTxBuffer; 
			if( bCommand < 100 ){
				if(!SyncTransceive()){
					bDevStatus = CM_TRANSFER_ERROR + ESPI1_bStatus;
					wRxSize = 2;
					if( !USB_WriteBuf( (BYTE*)&wRxSize, 2 ) ) return;
					if( !USB_WriteBuf( &bDevStatus, 1 ) ) return;
					// dumy for checksum	
					if( !USB_WriteBuf( &bDevStatus, 1 ) ) return;
				}else{
					if( !USB_WriteBuf( (BYTE*)&wRxSize, 2 ) ) return;
					if( !USB_WriteBuf( (BYTE*)wRxBuffer, wRxSize ) ) return;
				}
			}else{
				if(!AsyncTransceive()){
					bDevStatus = CM_TRANSFER_ERROR + ESPI1_bStatus;
					wRxSize = 2;
					if( !USB_WriteBuf( (BYTE*)&wRxSize, 2 ) ) return;
					if( !USB_WriteBuf( &bDevStatus, 1 ) ) return;
					// dumy for checksum	
					if( !USB_WriteBuf( &bDevStatus, 1 ) ) return;
				}else{
					if( !USB_WriteBuf( (BYTE*)&wRxSize, 2 ) ) return;
					if( !USB_WriteBuf( (BYTE*)wRxBuffer, wRxSize ) ) return;
				}
			}

		}
	}

	CM_Reply();
}

void CM_SetParam( void )
{
	CM_PARAM param; 

	if(!USB_Read( (BYTE*)&param, sizeof(CM_PARAM) )){
		CM_bStatus = CM_COMMAND_RX_TIMEOUT;
		return;
	}

	UT_StopTimer();
		
	CM_wCtrlTimeout = (WORD)(param.fCtrlTimeout*1000.0f);
	CM_wComTimeout = (WORD)(param.fComTimeout*1000.0f);
	USB_SetTimeout( CM_wComTimeout );
	ESPI1_SetTimeout( CM_wComTimeout );

	UT_SetTimer( param.dt * 1000.0f );

	CM_Reply();
}

void CM_GetParam( void )
{
	CM_PARAM param; 

	param.dt = UT_dt_ms / 1000.0f;
	param.fCtrlTimeout = (float)CM_wCtrlTimeout/1000.0f;
	param.fComTimeout  = (float)CM_wComTimeout /1000.0f;

	if(!USB_WriteBuf( (BYTE*)&param, sizeof(CM_PARAM) )){
		CM_bStatus = CM_REPLY_ERROR;
		return;
	}
	CM_Reply();
}

void CM_Reply( void )
{
	if(!USB_WriteBuf( &CM_bStatus, 1 )){
		CM_bStatus = CM_REPLY_ERROR;
		return;
	}
	if(!USB_Reply() ){
		CM_bStatus = CM_REPLY_ERROR;
		return;
	}		
}

